/*
*
* Copyright (C) 2000 Arne Schirmacher <arne@schirmacher.de>
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software Foundation,
* Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <iostream>
using std::cout;
using std::endl;

// GUI Page Handling Code
#include "kino_common.h"
#include "page.h"
#include "page_editor.h"
#include "page_capture.h"
#include "page_timeline.h"
#include "page_export.h"

#include "preferences.h"
#include "avi.h"
#include "filehandler.h"
#include "ieee1394io.h"
#include "message.h"
#include "jogshuttle.h"

extern "C"
{

#include <sys/stat.h>
#include <sys/time.h>
#include <gnome.h>
#include <math.h>

#include "callbacks.h"
#include "support.h"
#include "commands.h"
#include "jogshuttle.h"
#include "gtkenhancedscale.h"

	extern struct navigate_control g_nav_ctl;

	KinoCommon *common = NULL;

	static void getDroppedFiles( GtkWidget * w, GdkDragContext * context,
	                             gint x, gint y, GtkSelectionData * data, guint info,
	                             guint time, gpointer * extra )
	{

		if ( data->data )
		{
			gchar** urls = g_strsplit( ( gchar* ) data->data, "\n", 0 );
			gchar** tmp = urls;
			char *argv[ 2 ];

			/* For each URL we are passed */
			while ( tmp )
			{
				if ( strcmp( *tmp, "" ) == 0 )
					break;
				if ( !strncmp( *tmp, "file:", 5 ) )
				{
					argv[ 1 ] = *tmp + 5;
					argv[ 1 ][ strlen( argv[ 1 ] ) - 1 ] = '\0';
					common->bulkLoad( 2, argv );
				}
				tmp++;
			};
			g_strfreev( urls );
			windowMoved();
		}
	}

	// XXX: this is a workaround to a bug in libglade
	static void set_toolbar_tips( GtkWidget *widget )
	{
		GtkTooltips *toolbar_tips;
		toolbar_tips = gtk_tooltips_new();

		gtk_tooltips_set_tip( GTK_TOOLTIPS( toolbar_tips ),
			lookup_widget( widget, "button_new" ),
			_("New Playlist"), NULL );
		gtk_tooltips_set_tip( GTK_TOOLTIPS( toolbar_tips ),
			lookup_widget( widget, "button_open" ),
			_("Open a Video or Playlist"), NULL );
		gtk_tooltips_set_tip( GTK_TOOLTIPS( toolbar_tips ),
			lookup_widget( widget, "button_save" ),
			_("Save Playlist"), NULL );
		gtk_tooltips_set_tip( GTK_TOOLTIPS( toolbar_tips ),
			lookup_widget( widget, "button_prefs" ),
			_("Preferences"), NULL );
		gtk_tooltips_set_tip( GTK_TOOLTIPS( toolbar_tips ),
			lookup_widget( widget, "button_undo" ),
			_("Undo"), NULL );
		gtk_tooltips_set_tip( GTK_TOOLTIPS( toolbar_tips ),
			lookup_widget( widget, "button_redo" ),
			_("Redo"), NULL );
		gtk_tooltips_set_tip( GTK_TOOLTIPS( toolbar_tips ),
			lookup_widget( widget, "button_cut" ),
			_("Cut Scene"), NULL );
		gtk_tooltips_set_tip( GTK_TOOLTIPS( toolbar_tips ),
			lookup_widget( widget, "button_copy" ),
			_("Copy Scene"), NULL );
		gtk_tooltips_set_tip( GTK_TOOLTIPS( toolbar_tips ),
			lookup_widget( widget, "button_paste" ),
			_("Paste Before Frame"), NULL );
		gtk_tooltips_set_tip( GTK_TOOLTIPS( toolbar_tips ),
			lookup_widget( widget, "button_insert" ),
			_("Insert File Before Frame"), NULL );
		gtk_tooltips_set_tip( GTK_TOOLTIPS( toolbar_tips ),
			lookup_widget( widget, "button_append" ),
			_("Append File After Frame"), NULL );
		gtk_tooltips_set_tip( GTK_TOOLTIPS( toolbar_tips ),
			lookup_widget( widget, "button_split" ),
			_("Split Into Two Scenes Before Frame"), NULL );
		gtk_tooltips_set_tip( GTK_TOOLTIPS( toolbar_tips ),
			lookup_widget( widget, "button_join" ),
			_("Join Scene with Scene at Right"), NULL );
		gtk_tooltips_set_tip( GTK_TOOLTIPS( toolbar_tips ),
			lookup_widget( widget, "button_viewsize" ),
			_("Toggle between 50%, 100% and Auto Fill Views"), NULL );
		gtk_tooltips_set_tip( GTK_TOOLTIPS( toolbar_tips ),
			lookup_widget( widget, "button_help" ),
			_("Help"), NULL );
		gtk_tooltips_set_tip( GTK_TOOLTIPS( toolbar_tips ),
			lookup_widget( widget, "button_exit" ),
			_("Quit"), NULL );
	}
				  
	void kinoInitialise( GtkWidget * widget )
	{
		common = new KinoCommon( widget );
		common->setWindowTitle( );
		JogShuttle::getInstance();
		if ( ! Preferences::getInstance().enableV4L )
			gtk_notebook_remove_page( GTK_NOTEBOOK( lookup_widget( widget, "main_notebook" ) ), PAGE_BTTV );

		/* Allow drag from file manager */
		static GtkTargetEntry file_targets[] = {
		                                           {"text/uri-list", 0, 0}
		                                       };

		gtk_signal_connect( GTK_OBJECT( widget ), "drag_data_received",
		                    GTK_SIGNAL_FUNC( getDroppedFiles ), NULL );
		gtk_drag_dest_set( GTK_WIDGET( widget ),
		                   ( GtkDestDefaults ) ( GTK_DEST_DEFAULT_MOTION | GTK_DEST_DEFAULT_HIGHLIGHT |
		                                         GTK_DEST_DEFAULT_DROP ), file_targets, 1,
		                   ( GdkDragAction ) ( GDK_ACTION_COPY | GDK_ACTION_MOVE ) );
		set_toolbar_tips( widget );
		gtk_window_set_default_size( GTK_WINDOW( widget ),
			Preferences::getInstance().windowWidth, Preferences::getInstance().windowHeight );
	    gtk_paned_set_position( GTK_PANED( lookup_widget( widget , "hpaned1" ) ),
			Preferences::getInstance().storyboardPosition );
		common->setPreviewSize( Preferences::getInstance().previewSize, true );
	}

	gboolean kinoDeactivate()
	{
		if ( common->exitKino( ) )
		{
			cerr << "Exiting Kino\n";
			gtk_window_get_size( GTK_WINDOW( common->getWidget() ), &Preferences::getInstance().windowWidth,
				&Preferences::getInstance().windowHeight );
			Preferences::getInstance().storyboardPosition = gtk_paned_get_position( GTK_PANED(
				lookup_widget( common->getWidget() , "hpaned1" ) ) );

			Preferences::getInstance().Save();
			delete common;
			gtk_exit( 0 );
		}
		return TRUE;
	}

	void refresh( )
	{
		cout << "Refresh\n";
		//common->moveToFrame( );
		//gtk_widget_queue_draw( lookup_widget( common->getWidget(), "drawingarea1") );
	}

	void bulkLoad( int argc, char * argv[] )
	{
		common->bulkLoad( argc, argv );
	}

	void newFile( )
	{
		common->newFile();
	}

	void openFile( )
	{
		common->loadFile();
	}

	void savePlayListAs( )
	{
		common->savePlayListAs();
	}

	void savePlayListAsEli( )
	{
		common->savePlayListAsEli();
	}

	void savePlayList( )
	{
		common->savePlayList();
	}

	void saveFrame( )
	{
		common->saveFrame();
	}

	void insertFile( )
	{
		common->insertFile();
	}

	void appendFile( )
	{
		common->appendFile();
	}

	void pageStart( int page )
	{
		common->setCurrentPage( page );
	}

	int moveToFrame( int frame )
	{
		return common->moveToFrame( frame );
	}

	void videoStart( )
	{
		common->videoStartOfMovie();
	}

	void videoPreviousScene( )
	{
		common->videoPreviousScene();
	}

	void videoStartOfScene( )
	{
		common->videoStartOfScene();
	}

	void videoRewind( )
	{
		common->videoRewind();
	}

	void videoBack( )
	{
		common->videoBack();
	}

	void videoPlay( )
	{
		common->videoPlay();
	}

	void videoForward( )
	{
		common->videoForward();
	}

	void videoFastForward( )
	{
		common->videoFastForward();
	}

	void videoNextScene( )
	{
		common->videoNextScene();
	}

	void videoEndOfScene( )
	{
		common->videoEndOfScene();
	}

	void videoEndOfMovie( )
	{
		common->videoEndOfMovie();
	}

	void videoStop( )
	{
		common->videoStop();
	}

	void videoPause( )
	{
		common->videoPause();
	}

	void videoShuttle( int angle )
	{
		common->videoShuttle( angle );
	}

	void windowMoved()
	{
		if ( common )
			common->windowMoved();
	}

	void visibilityChanged( gboolean visible )
	{
		common->visibilityChanged( visible );
	}

	void notebookChangePage( int page )
	{
		common->changePageRequest( page );
	}

	gboolean processKeyboard( GdkEventKey * event )
	{
		// To avoid problems with repeat keys only process
		// top of the event queue key presses - issue is that
		// we don't know if any pending event is actually a
		// keypress... think all key press should move over to
		// a snoop, but even then.. dunno if that'll help
		if ( Preferences::getInstance().disableKeyRepeat && gdk_events_pending() )
			return FALSE;

		return common->processKeyboard( event );
	}

	void processCommandLine( const char * command )
	{
		common->processCommandLine( command );
	}

	void processMenuCommand( char * command )
	{
		common->processMenuCommand( command );
	}

	void selectScene( int i )
	{
		common->selectScene( i );
	}

	void startCapture( void )
	{
		common->getPageCapture() ->startCapture();
	}

	void stopCapture( void )
	{
		common->getPageCapture() ->stopCapture();
	}

	void setPreviewSize( float factor )
	{
		common->setPreviewSize( factor );
	}

	void previewExport( void )
	{
		common->getPageExport() ->previewExport();
	}

	void startExport( void )
	{
		common->getPageExport() ->startExport();
	}
	void stopExport( void )
	{
		common->getPageExport() ->stopExport();
	}

	void pauseExport( void )
	{
		common->getPageExport() ->pauseExport();
	}

	void setExportMode( int mode )
	{
		common->getPageExport() ->setCurrentMode( mode );
	}

	void RefreshBar( GtkWidget * drawingarea )
	{
		common->getPageEditor() ->DrawBar( common->g_currentFrame );
	}

	void setMoreInfo( int state )
	{
		common->setMoreInfo( state == 1 );
		Frame &frame = *( GetFramePool( ) ->GetFrame( ) );
		common->showFrameMoreInfo( frame, NULL );
		GetFramePool( ) ->DoneWithFrame( &frame );
		common->moveToFrame();
	}

	void setTimeFormat( int format )
	{
		common->setTimeFormat( format );
	}

} // extern "C"
